#! /usr/bin/perl -w
#
# fireparse by Aaron D. Marasco (Aaron@Marasco.com) - No warranties expressed nor implied.
#  PLEASE send me your comments! Please read until "end of user serviceable" below.
#
# Please see the included README file for proper iptables setup for use with fireparse.
#
# 2.2 ADM -  9 Mar 2001 - Added "subject" tags -DATE-, -ENTRIES-, -MAX-
#           11 Mar 2001 - Re-merged iptables and ipchains code branches!
#           12 Mar 2001 - Allow user to decide UID (thanx Jeff)
#                       - Services files can be missing now (I was never happy with that)
#           15 Mar 2001 - -MAX- bug fix (0 is not nice)
# 2.1 ADM - 24 Feb 2001 - Code cleanup
#           25 Feb 2001 - System calls added for gethostbyaddr() and hostname caching (MUCH less overhead)
#                       - Most config data moved to constants for optimization
#           28 Feb 2001 - Slight security fix
#            3 Mar 2001 - Added clean-up expression 'cleanex', some code clean-up
#	     5 Mar 2001 - ICMP fix and local file output fix (2x thanks to Jimmie Yoo!)
# 2.0 - ADM - 6 Jan 2001 - Parsing rewrites for Netfilter/IPTables
#
# Known Bugs/Concerns:
#   - Not too happy with the way things have to be with the IPTables LOG extension. Maybe somebody will rewrite ipt_LOG for me. ;)
#   - Text mode alignment may be off if the chain name is more than 8 characters. I think everyone uses HTML output anyway...
#   *** Log prefixes with !'s in them WILL fail. ***
#

# The setup section follows. It is broken up into four sections:
#  NORMAL - the only section that MOST users will need to configure
#  CUSTOM - customization of how fireparse runs (defaults should be fine)
#  DISTRO - distribution-specific, defaulting to RedHat / System V setup
#  ADVANCED - for advanced users only

### NORMAL ###
# E-mail report address (Use single-quotes)
my $mailto = 'root@localhost';
# Do you use a HTML smart e-mail reader? If so, this will allow nicely formatted info:
use constant htmlon => 1; # Set to 1 also if you use local_file output (in CUSTOM section)
# Are you using 'ipchains' or 'iptables' for your kernel firewall? If 'iptables' then set to 1, 'ipchains' set to 0.
use constant use_iptables => 1;
# Your internal network (Use single-quotes)
# Example: '192.168.1.' will match the 192.168.1.nnn subnet
use constant internalnet => '192.168.1.';
### ENDNORMAL ###

### CUSTOM ###
# Your 'date' command. Set the display options like you want from 'man date'. (Thanx PE)
# You must leave the chomp() around it or the e-mail will not work.
my $date;
chomp($date = `/bin/date +%m-%d-%Y`); # Default of %m-%d-%Y is more "American" but also sorts by subject nicer IMHO.
# Backup log - if you want DAILY files, comment next line and uncomment one after.
my $FireLog = "/var/log/messages.fire";
# my $FireLog = "/var/log/messages.fire-$date"; # (PE)
# 1 enables reverse name lookup.
use constant hostcmd_do => 1;
# 1 enables internal address. This shows the address on the internal network involved.
use constant local_ip_do => 1;
# Output type. If set to '1' then set $local_file_out to the BASE file name. '1' will NOT mail report
#  but will create a local HTML file named BASE-DATE.html (see above for settable $date format)
use constant local_file => 0;
my $local_file_out = "/home/httpd/html/logs/fire-$date.html"; # will output /home/httpd/html/logs/fire-10-08-2000.html
# What user should we run as? ('nobody' is recommended)
use constant userid => 'nobody';
# Subject of outgoing mail message - available variables are -DATE-, -ENTRIES-, -MAX-  == Date, Lines reported, Max matches seen
# my $mail_subject = "Firewall Report for -DATE- - -ENTRIES- lines";
my $mail_subject = "Firewall Report for -DATE- - -ENTRIES- lines (-MAX- max)";
# Title of web page
my $html_TITLE_val = "Firewall Report";
# Table header
my $html_tablehdr = "Automated Firewall Packet Log Report";
### ENDCUSTOM ###

### DISTRO ###
# Your outgoing mail program (the default works with qmail also)
my $mail = "/usr/sbin/sendmail -t";
# Which log file
my $MainLog = "/var/log/messages";
# Your services file (must exist)
use constant servicefile => "/etc/services";
# Extended services file (see example).
use constant servicefile2 => "/etc/services.fake";
# Set to '1' if you use a single script to start/stop your logging services
#  (RedHat / SunOS / SysV style) vs BSD style
# The script should take the parameter "start" or "stop".
use constant SysVStyle => 1;
# Your startup script for "syslogd". (Used if SysVStyle is 1)
my $syslog_cmd = "/etc/rc.d/init.d/syslog";
# Command to START if SysVStyle is 0 (check /etc/rc.d/rc.M)
my $syslog_bsd_strt = "/usr/sbin/syslogd ; /usr/sbin/klogd -c 3";
# Command to STOP if SysVStyle is 0
sub syslog_bsd_stop {
	my $pid;

	open PID, "/var/run/syslogd.pid" or die "Unable to open pid file for syslogd: $!\n";
	chomp ($pid = <PID>);
	close PID;
	kill 9, $pid;

	open PID, "/var/run/klogd.pid" or die "Unable to open pid file for klogd: $!\n";
	chomp ($pid = <PID>);
	close PID;
	kill 9, $pid;
};
### ENDDISTRO ###

### ADVANCED ###
# HTML Tags. DO NOT MODIFY unless you know HTML! Escape quotes with backslash. You can add background images, etc here:
my $html_HEAD = "<HEAD>";
my $html_BODY = "<BODY BGCOLOR=\"white\" text=\"black\">"; # I know this is depreciated in HTML 4.0
my $html_color = "F06060"; # The background color of DROP and REJECT packets.
# Log file cleanup (alpha stage)
# If there are other useless lines in the log file you want to wipe while it is offline anyway, insert as many regular
# expressions as you like here. If you don't know what I am talking about, ignore this line.
my $cleanex;
# $cleanex = q{(?: ppp(?:d|oe)\[\d+\]|httpd s.* succeeded$)}; # Default gets rid of pppd and pppoe messages [I use net-check to watch that ;) ]
# print STDERR "cleanex = $cleanex \n"; exit;
### ENDADVANCED ###

#
#### End of User Serviceable Parts. Feel free to read on and comment tho.
#

use strict;
require Socket; # Socket is in the standard perl distro, so should be OK.
use Socket;
require File::Copy; # Again, in default distro
use File::Copy;

#
## WARNING: THIS PART IS KINDA DANGEROUS. ;)
#
print STDOUT "Opening the log file...\n";
if (SysVStyle == 1) {
	`$syslog_cmd stop > /dev/null 2> /dev/null`;
} else {
	&syslog_bsd_stop;
};
# Steal the system log.
move ("$MainLog", "$MainLog.fireparsing");
# Open files
open MAINLOG, "$MainLog.fireparsing" or die "Could not open the renamed file! [WARNING: syslog may be STOPPED!] - $!";
open NOTMYLOG, ">$MainLog" or die "Could not recreate log! [WARNING: syslog may be STOPPED!] - $!";
# Exclusive lock files
flock MAINLOG, 2 or die "Could not lock the (renamed) file! [WARNING: syslog may be STOPPED!] - $!";
flock NOTMYLOG, 2 or die "Could not lock the (new) file! [WARNING: syslog may be STOPPED!] - $!";
# Pull out our lines
print STDOUT "Parsing... ";
my @filterlines; # Must declare outside of while loop
my $lookfor = (use_iptables?'kernel: fp=':'kernel: Packet log:'); # Set what we are looking for.
while (<MAINLOG>) {
	if ( /$lookfor/o ) {
		# This is a line we want to keep
		chomp;
		push @filterlines, $_;
	} else {
		# Strip or ignore this line
		if ((defined $cleanex) && (/$cleanex/o)) {
			# Trash this line
			# print STDERR "Cleanex wiped: $_";
		} else {
			print NOTMYLOG;
		}
	} # If ours
} # Parsing file
flock NOTMYLOG, 8; #unlocks file
close NOTMYLOG;
# Tell syslogd to reopen log file
if (SysVStyle == 1) {
	`$syslog_cmd start > /dev/null 2> /dev/null`;
} else {
	`($syslog_bsd_strt) > /dev/null 2> /dev/null`;
};
## END OF "DANGEROUS" STUFF
close MAINLOG;
unlink "$MainLog.fireparsing";
open FIREOUT, ">>$FireLog";
# Switch to 'userid' (no need to be root anymore)
$< = getpwnam(userid);
$> = getpwnam(userid);
if (scalar @filterlines == 0) { die "Nothing to do.\n"; }
# At this point we have all the relevant lines in the array @filterlines
# Parse the services file. This will give us a hash with the name of all the port services
print STDOUT scalar @filterlines . " relevant lines.\nExamining services... ";
# Only relevant info for us are first two fields in services
my @service; # Must declare outside of while loops
my %services; # Must declare outside of while loops
if (open SERVICEFILE, servicefile) {
	while (<SERVICEFILE>) {
		chomp; # remove newline
		@service = split; # parse and remove whitespace
		if (scalar @service >= 2) {
			if (($service[0] !~ "#") and ((length $service[0]) > 2) and ($service[1] =~ "/") ) {
				# Not a comment or a blank line
				$services{$service[1]} = $service[0]; # name now stored in hash
			} # not a comment
		} # not blank line
	} # end of file
	close SERVICEFILE;
} # File #1 exists or not
if (open SERVICEFILE, servicefile2) {
	while (<SERVICEFILE>) {
		chomp; # remove newline
		@service = split; # parse and remove whitespace
		if (scalar @service >= 2) {
			if (($service[0] !~ "#") and ((length $service[0]) > 2) and ($service[1] =~ "/") ) {
				# Not a comment or a blank line
				$services{$service[1]} = $service[0]; # name now stored in hash
			} # not a comment
		} # not blank line
	} # end of file
	close SERVICEFILE;
} # File #2 exists or not
print STDOUT (scalar keys %services)." services registered.\n";
print STDOUT "Sorting and counting... ";
# Now process @filterlines into the format we want it (both ipchains and iptables use same internals now). This line is beautiful. ;)
@filterlines = @{use_iptables ? &iptables_parse(\@filterlines) : &ipchains_parse(\@filterlines)};
close FIREOUT;
@filterlines = sort @filterlines;
# OK, now it is sorted. Now to create a new array with only unique lines BUT also appended to the end
#  the number of lines originally seen
my @filteredlines;
$filteredlines[0] = shift @filterlines; # initialize the new list with the first line
my $discard = 0;
my $current = 0;
my $count = 1;
my $maxcount = 1;
while (scalar @filterlines != 0) {
	my $tempoline = shift @filterlines;
	if ($tempoline eq $filteredlines[$current]) { # Line matches previous
		$count++;
		$maxcount = $count if ($count > $maxcount);
	} else { # New match
		$filteredlines[$current] = $filteredlines[$current]."!".$count; # Store off how many there were
		$count = 1; # Re-initialize
		$current++; # Prepare for new value
		$filteredlines[$current] = $tempoline; # Store this unique line
	} # End of compare
} # Now @filterlines is empty and @filteredlines is full
$filteredlines[$current] = $filteredlines[$current]."!".$count; # Store off how many there were for last value
my $entries = (scalar @filteredlines);
print STDOUT "$entries 'unique' unverified entries.\n";
# Open a mail message - first fill in tag fields
if ($mail_subject =~ /(.*)-DATE-(.*)/) { $mail_subject = $1.$date.$2; };
if ($mail_subject =~ /(.*)-ENTRIES-(.*)/) { $mail_subject = $1.$entries.$2; };
if ($mail_subject =~ /(.*)-MAX-(.*)/) { $mail_subject = $1.$maxcount.$2; };
if (local_file == 1) {
	open (NewMsg, ">$local_file_out") or warn "\nCannot open file handle.\n";
} else { # Don't want the e-mail headers in the HTML files!
	open (NewMsg, "|$mail") or warn "\nCannot open mail handle.\n";
	print NewMsg <<HEADER;
To: $mailto
Subject: $mail_subject
MIME-Version: 1.0
Content-Type: text/html; charset=\"us-ascii\"

HEADER
};
if (htmlon == 1) {
	print NewMsg <<HTMLHEADER;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">
<HTML>$html_HEAD<META CREATOR="Aaron D. Marasco"><TITLE>$html_TITLE_val</TITLE></HEAD>
$html_BODY<TABLE WIDTH=95% BORDER=1><TH COLSPAN=9>$html_tablehdr</TH><TR><TD ALIGN=CENTER>
HTMLHEADER
	if (local_ip_do == 1) {
		print NewMsg "<B>Date</B></TD><TD ALIGN=CENTER><B>Chain</B></TD><TD ALIGN=CENTER><B>Rule</B></TD><TD ALIGN=CENTER><B>Action</B></TD><TD ALIGN=CENTER><B>Qty</B></TD><TD ALIGN=CENTER><B>Ext. Address</B></TD><TD ALIGN=CENTER><B>Int. Address</B></TD><TD ALIGN=CENTER><B>Ext Protocol</B></TD><TD ALIGN=CENTER><B>Int Protocol</B></TD></TR><TR><TD>";
	} else {
		print NewMsg "<B>Date</B></TD><TD ALIGN=CENTER><B>Chain</B></TD><TD ALIGN=CENTER><B>Rule</B></TD><TD ALIGN=CENTER><B>Action</B></TD><TD ALIGN=CENTER><B>Qty</B></TD><TD ALIGN=CENTER><B>Ext. Address</B></TD><TD ALIGN=CENTER><B>IFace</B></TD><TD ALIGN=CENTER><B>Ext Protocol</B></TD><TD ALIGN=CENTER><B>Int Protocol</B></TD></TR><TR><TD>";
	}
} else {
	print NewMsg "\n\n";
}
# Now to parse each line individually to see who was who (source vs. destination). Also pull port
#  number and cross-reference with services. (maybe some day nslookup calls can go here)
# This includes final output cuz we don't keep running over and over on the loops.
my ($date1, $date2, $chain, $rule, $action, $addy, $intf, $thisservice, $thatservice, $exthost, %hostcache);
# Set up the display
format STDOUT_TOP =                                                           #
			       FIREWALL REPORT
Date    Chain     Rule  Action  Qty    Ext. Address     IFace  Protocol
------------------------------------------------------------------------------
.
format STDOUT =
@<< @<  @<<<<<<<  @<<<  @<<<<<  @>>>>  @<<<<<<<<<<<<<<  @<<<<  @<<<<<<<<<<<<<<
$date1, $date2, $chain, $rule, $action, $count, $addy, $intf, $thisservice
				       @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
				       $exthost
.
#MMM DD  12345678  9999  MASQ    99999  999.999.999.999  ppp99  kerberos_master
format NewMsg_TOP =
				       FIREWALL REPORT
Date    Chain     Rule  Action  Qty    Ext. Address     IFace          Protocols
---------------------------------------------------------------EXT--------------INT----------
.
format NewMsg =
@<< @<  @<<<<<<<  @<<<  @<<<<<  @>>>>  @<<<<<<<<<<<<<<  @<<<<  @<<<<<<<<<<<<<<  @<<<<<<<<<<<<<<
$date1, $date2, $chain, $rule, $action, $count, $addy,  $intf, $thisservice,    $thatservice
				       @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
				       $exthost
.
for (my $item = 0; $item < (scalar @filteredlines); $item++) { # scalar is 1 based but loop/index is 0 based
	my ($proto, $addy1, $addy2, $port, $port2, $direction, $local_ip);
	($date1, $date2, $chain, $action, $intf, $proto, $addy1, $addy2, $rule, $count) = split (/!/, $filteredlines[$item]);
	# print STDERR "@filteredlines[$item] => addy1(raw): $addy1, addy2(raw): $addy2\n";
	if ($addy1 !~ internalnet) { # addy1 is the EXTERNAL one
		($addy, $port) = split (/:/, $addy1);
		($local_ip, $port2) = split (/:/, $addy2);
		$direction = "&lt;- ";
	} else { # addy2 must be the external one
		($addy, $port) = split (/:/, $addy2);
		($local_ip, $port2) = split (/:/, $addy1);
		$direction = "-&gt; ";
	} # now addy and port set correctly

	# Trying not to bog down too much, we will just check address stuff.
	# Check if addy is "some numbers . some numbers . some numbers . some numbers"
	# The following stricter regex is new (v1.7) from O'Reilly Mastering Regular Expressions
	if ($addy !~ /^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])$/o) {
		# print STDERR "MANGLE: addy prob: $addy\n";
		$discard++;
		next;
	}
	# And repeat for internal (v1.8)
	if ($local_ip !~ /^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])$/o) {
		$discard++;
		# print STDERR "MANGLE: local_ip prob: $local_ip\n";
		next;
	}
	# Check the ports for a number between 0 and 65535.
	if (($port < 0) || ($port > 65535) || ($port eq "")) {
		$discard++;
		# print STDERR "MANGLE: port prob: $port\n";
		next;
	}
	if (($port2 < 0) || ($port2 > 65535) || ($port2 eq "")) {
		# print STDERR "MANGLE: port2 prob: $port2\n";
		$discard++;
		next;
	}
	# End of check. If it made it this far...
	$thisservice = $port."/".$proto;
	$thatservice = $port2."/".$proto;
	if (defined $services{$thisservice}) {
		$thisservice = $services{$thisservice};
	} # Now $thisservice should be either text or port/proto - no matter
	if (defined $services{$thatservice}) {
		$thatservice = $services{$thatservice};
	} # Now $thatservice should be either text or port/proto - no matter
	# Do a reverse DNS lookup
	if (hostcmd_do == 1) {
		if ( defined $hostcache{$addy} ) { # Cache hit (new in 2.1, makes MUCH faster!)
			$exthost = $hostcache{$addy};
			# print STDERR "Cache hit for ".$addy." ==> ".$exthost."\n";
		} else { # Cache miss
			my $tempo = gethostbyaddr (inet_aton($addy), Socket::AF_INET);
			if (defined ($tempo)) { # valid, set and save to cache
				$exthost = $tempo;
				$hostcache{$addy} = $exthost;
			} else { # Not valid
				$exthost = "Host not found.";
			} # gethostbyaddr result
		} # Cache
	} else { # hostcmd_do
		$exthost = "Host not examined.";
	} # hostcmd_do
	write STDOUT; # output the report line
	if (htmlon == 0) {
		write NewMsg; # Append to the mail
	} else {
		use constant HCOL => "</TD><TD>"; # Shorthand for new col in HTML table
		$addy = $direction.$addy;
		print NewMsg $date1, " ", $date2, HCOL, $chain, HCOL, $rule, "</TD>";
		if (($action =~ "DROP") || ($action =~ "REJECT")) {
			print NewMsg "<TD BGCOLOR=\"$html_color\">";
		} else {
			print NewMsg "<TD>";
		}
		if (local_ip_do == 1) {
			print NewMsg $action, HCOL, $count, HCOL, $addy, HCOL, $local_ip." (".$intf.")", HCOL, $thisservice, HCOL, $thatservice, "</TD></TR>\n<TR><TD>";
		} else {
			print NewMsg $action, HCOL, $count, HCOL, $addy, HCOL, $intf, HCOL, $thisservice, HCOL, $thatservice, "</TD></TR>\n<TR><TD>";
		}
		print NewMsg (HCOL)x4, "</TD><TD COLSPAN=2>", $exthost, "</TD></TR><TR><TD>" if ((hostcmd_do == 1) && (!($exthost =~ "Host not")));
	} # end of HTML output
} # end of array
print NewMsg "</TD></TR></TABLE>\n" if htmlon;
if ($discard != 0) {
	print NewMsg "WARNING: ".$discard." entries were mangled, and therefore ignored.\n";
	print STDOUT "WARNING: ".$discard." entries were mangled, and therefore ignored.\n";
}
print NewMsg "<BR></BODY></HTML>" if htmlon;
# Send the e-mail
close (NewMsg);

# End of script &main()

sub iptables_parse {
	my @lines = @{$_[0]}; # We're given a reference to an array.
	# Begin to parse all the lines.
	# Some examples (can greatly vary):
	#Jan  5 00:15:32 ragnarok kernel: fp=Test:1 a=DROP IN=eth1 OUT=ppp0 SRC=192.168.1.69 DST=192.147.174.209 LEN=48 TOS=0x00 PREC=0x00 TTL=127 ID=52511 DF PROTO=TCP SPT=1620 DPT=80 WINDOW=8192 RES=0x00 SYN URGP=0
	#Jan  5 00:15:32 ragnarok kernel: fp=Test:1 a=DROP IN=eth1 OUT=ppp0 SRC=192.168.1.69 DST=192.147.174.209 LEN=55 TOS=0x00 PREC=0x00 TTL=127 ID=52511 DF PROTO=TCP SPT=1620 DPT=80 WINDOW=8192 RES=0x00 SYN URGP=0
	#Jan  6 14:21:02 ragnarok kernel: fp=DNS:3 a=ACCEPT IN=ppp0 OUT= MAC= SRC=209.249.164.210 DST=141.157.88.148 LEN=124 TOS=0x00 PREC=0x00 TTL=50 ID=3186 PROTO=UDP SPT=53 DPT=1029 LEN=104
	#Jan  6 14:21:02 ragnarok kernel: fp=DNS:3 a=ACCEPT IN=ppp0 OUT= MAC= SRC=209.249.164.210 DST=141.157.88.148 LEN=140 TOS=0x00 PREC=0x00 TTL=50 ID=3190 PROTO=UDP SPT=53 DPT=1029 LEN=120
	#Jan  6 14:42:44 ragnarok kernel: fp=ICMP:1 a=DROP IN=eth0 OUT= MAC=00:b0:d0:22:67:db:00:50:0f:fe:7e:70:08:00 SRC=63.240.56.11 DST=24.162.0.105 LEN=84 TOS=0x00 PREC=0x00 TTL=56 ID=22641 PROTO=ICMP TYPE=8 CODE=0 ID=24634 SEQ=0
	# First the line is repacked with only the information we want.
	my (@chain, $item); # Must declare outside of for loop
	for ($item = 0; $item < (scalar @lines); $item++) { # scalar is 1 based but loop/index is 0 based
		print FIREOUT $lines[$item]."\n"; # Save off the original for the user
		my @sl = split (/\s+/, $lines[$item]); # sl short for splitline
		# The only real definites here are everything to the left of 'kernel'
		my %logline; # There will be a hash 'logline' containing all the data from the line. Then we pick and choose.
		$logline{'MON'}=$sl[0];
		if ($sl[1] < 10) { # Want to insert a zero into day
			$logline{'DAY'}="0".$sl[1];
		} else {
			$logline{'DAY'}=$sl[1];
		}
		# This is all we want from 0 thru 4 in @sl.
		splice(@sl, 0, 5); # Cut out Date/Date/Time/Hostname/'kernel'
		# Some defaults
		$logline{'IN'} = "dunno";
		$chain[1] = "?";
		# Almost everything that remains has equal signs on it (except some TCP flags like DF and SYN)
		my $dataword;
		foreach $dataword (@sl) {
			my @pair = split (/=/, $dataword);
			if (scalar @pair == 2) {
				$logline{$pair[0]} = $pair[1];
			}
		}
		# Now combine them to look like the old info from pre-2.0 (code reuse!)
		@chain = split (/:/, $logline{'fp'});
		$logline{'PROTO'} = lc $logline{'PROTO'};
		if ($logline{'PROTO'} eq "icmp") { # Thanx JY!
			$logline{'SPT'} = 0;
			$logline{'DPT'} = $logline{'TYPE'};
		}
		$lines[$item] = $logline{'MON'}."!".$logline{'DAY'}."!".$chain[0]."!".$logline{'a'}."!".$logline{'IN'}."!".$logline{'PROTO'}."!".$logline{'SRC'}.":".$logline{'SPT'}."!".$logline{'DST'}.":".$logline{'DPT'}."!".$chain[1];
		# Date, date, chain, result, interface, protocol, source, detination, rule number
	} # Done repacking the line
	return \@lines;
}

sub ipchains_parse {
	my @lines = @{$_[0]}; # We're given a reference to an array.
	# First the line is repacked with only the information we want.
	my $item; # Must declare outside of for loop
	for ($item = 0; $item < (scalar @lines); $item++) { # scalar is 1 based but loop/index is 0 based
		print FIREOUT $lines[$item]."\n";
		my @sl = split (/\s+/, $lines[$item]); # sl short for splitline
		my @proto = split (/=/, $sl[10]); # strips off the "PROTO="
		if ($proto[1] == 6) { # known protocols for /etc/services
			$proto[1] = "tcp";
		} elsif ($proto[1] == 17) {
			$proto[1] = "udp";
		} elsif ($proto[1] == 1) {
			$proto[1] = "icmp";
		}
		if ($sl[18] eq "SYN") {  # Remove the SYN flag
			splice (@sl, 18, 1);
		}
		if ($sl[1] < 10) { # Want to insert a zero
			$lines[$item] = $sl[0]."!0".$sl[1]."!".$sl[7]."!".$sl[8]."!".$sl[9]."!".$proto[1]."!".$sl[11]."!".$sl[12]."!".$sl[18];
			# Date, date, chain, result, interface, protocol, source, detination, rule number
		} else {
			$lines[$item] = $sl[0]."!".$sl[1]."!".$sl[7]."!".$sl[8]."!".$sl[9]."!".$proto[1]."!".$sl[11]."!".$sl[12]."!".$sl[18];
		}; # done shortening line
	}
	return \@lines;
}
